%
% compare krylov_spectral.m with irbleigs.m and arpack eigs on some
% realistic model problems.
%

global A

for id = 1: 3

  switch id
   case 1,
    A = model(42, 80); 
  
   case 2,
    A=mmread('SiH4_sym.mtx');
    fprintf(' model: A = SiH4 \n')

   case 3,
    A=mmread('Si10H16_sym.mtx');
    fprintf(' model: A = Si10H16 \n')

  end
  
ndim = size(A,1);

orthF = 0;
max_restart = 2500;
tol = 1e-12;


for itt =  2 : 4
  % nneed = 30*itt;
  nneed = 20*itt;
  
  blksize = 4;
  k = floor((nneed+30)/blksize);

  m = min(3*k, ndim/10);

  F0 = rand(ndim, blksize);  
  F = F0;
  fprintf('==> begin krylov-spectral\n');

  opts = struct('tol', tol, 'blksize', 4, 'maxvsize', m, 'nkeep', k);
  t1 = cputime;
  [eigval, V, nconv, T, info] = ks(nneed, F0, opts);

  ks_cputime = cputime - t1;

  fprintf('\nkrylov-spectral:  ndim = %i,  blksize = %i,  nneed=%i,   nconv=%i\n', ...
	  ndim, blksize, nneed, nconv);
  
  %% verify the actual residual norms
  F=A*V(:,1:nconv) - V(:,1:nconv)*diag(eigval(1:nconv));
  for i = 1:nconv
    errnrm(i) = norm(F(:,i))/abs(eigval(i));
  end
  fprintf('krylov-spectral:  actual residual norms  ||A*v-lambda*v||/|lambda|\n');
  fprintf('%11.5e   %11.5e   %11.5e   %11.5e   %11.5e\n', errnrm(1:nconv))
  [ernrm_max_ks, idx] = max(abs(errnrm(1:nconv)));
  fprintf('krylov-spectral max error norm:  errnrm( %i ) = %e\n\n', idx, ernrm_max_ks);


  if (1==1),
    fprintf('==> call IRBLEIG for comparision, may take a long time...\n');   
    OPTS = struct('BLSZ', blksize, 'K', nneed, 'SIGMA', 'SE', 'V0', F0, ...
		  'MAXIT', max_restart, 'TOL', tol);
    V=[];
    tBL = cputime;
    [V,D] = irbleigs(A, OPTS);
    irbl_cputime =cputime - tBL;
    
    nd = size(D,1);
    fprintf('\nirbleigs:  ndim = %i,   nneed=%i,   nconv=%i\n', ndim, nneed, nd);
  
    if (nd > 0),
      %% verify the actual residual norms
      F=A*V(:,1:nd) - V(:,1:nd)*D;
      for i = 1:nd
	errnrm2(i) = norm(F(:,i))/abs(D(i,i));
      end
      fprintf('irbleigs:  actual residual norms  ||A*v-lambda*v||/lambda\n');
      fprintf('%11.5e   %11.5e   %11.5e   %11.5e   %11.5e\n', errnrm2(1:nd))
      [ernrm_max_irbl, idx] = max(abs(errnrm2(1:nconv)));
      fprintf('irbleigs max error norm:  errnrm( %i ) = %e\n\n', idx, ernrm_max_irbl); 
    else
       fprintf('**** irbleigs does not converge in %i iterations ****\n', max_restart);
    end
    speedgain=irbl_cputime/ks_cputime;
    fprintf('ks_max_errnrm=%e, irbl_max_errnrm=%e\n', ernrm_max_ks, ernrm_max_irbl); 
    fprintf('ks_cputime=%e,    irbl_cputime=%e,   (irbl_cpu/ks_cpu)=%e\n', ...
            ks_cputime, irbl_cputime, speedgain);  

    fprintf('\n');
  end
  
   if (1==1),
    OPTS = struct('issym', 1, 'isreal', 1, 'tol', tol, 'v0', F0(:,1), 'maxit', max_restart, 'disp', 0);
    V=[]; D=[];

    teigs = cputime;
    [V,D] = eigs(A, nneed, 'SA', OPTS);
    eigs_cputime =cputime - teigs;
    
    nd = size(D,1);
    fprintf('\narpack eigs:  ndim = %i,  blksize= %i,  nneed=%i,   nconv=%i\n', ...
             ndim,  blksize, nneed, nd);
  
    if (nd > 0),
      %
      % verify the actual residual norms
      %
      F=A*V(:,1:nd) - V(:,1:nd)*D;
      for i = 1:nd
	errnrm3(i) = norm(F(:,i))/abs(D(i,i));
      end
      fprintf('arpack eigs:  actual residual norms  ||A*v-lambda*v||/|lambda|\n');
      fprintf('%11.5e   %11.5e   %11.5e   %11.5e   %11.5e\n', errnrm3(1:nd))
      [ernrm_max_arpack, idx] = max(abs(errnrm3(1:nconv)));
      fprintf('arpack max error norm:  errnrm( %i ) = %e\n\n', idx, ernrm_max_arpack);    
    else
       fprintf('arpack_eigs does not converge in %i iterations\n', max_restart);
    end

    speedgain2=eigs_cputime/ks_cputime;
    fprintf('ks_max_errnrm=%e,     arpack_max_errnrm=%e\n', ...
           ernrm_max_ks,  ernrm_max_arpack);  
    fprintf('ks_cputime=%e,     eigs_cputime=%e,   arpack_cpu/ks_cpu=%e\n\n', ...
            ks_cputime,   eigs_cputime, speedgain2);  
  

     blksize = nneed;
     F = V(:,1:nneed)+ tol*3*rand(ndim, nneed);
     %F = V(:,1:nneed);
     k = floor((nneed+30)/blksize);
     m = min(2*k, ndim/10);
     blkcontrol=-1;
  
     fprintf('==> begin krylov-spectral with good initial guesses\n');

     t1 = cputime;
     [eigval, V, nconv, T, info] = ks(nneed, F, opts);
     ks_cputime2 = cputime - t1;

     fprintf('\nkrylov-spectral:  ndim = %i,   nneed=%i,   nconv=%i\n', ndim, nneed, nconv);
  
     %% verify the actual residual norms
     F=A*V(:,1:nconv) - V(:,1:nconv)*diag(eigval(1:nconv));
     for i = 1:nconv
       errnrm(i) = norm(F(:,i))/abs(eigval(i));
     end
     fprintf('krylov-spectral:  actual residual norms  ||A*v-lambda*v||/|lambda|\n');
     fprintf('%11.5e   %11.5e   %11.5e   %11.5e   %11.5e\n', errnrm(1:nconv))
     [ernrm_max_ks, idx] = max(abs(errnrm(1:nconv)));
     fprintf('krylov-spectral max error norm:  errnrm( %i ) = %e\n\n', idx, ernrm_max_ks);
 
     speedgain3 =  eigs_cputime/ ks_cputime2;
     speedgain4 =  ks_cputime / ks_cputime2;
     fprintf('ks_cputime=%e,   ks_cputime2=%e,   eigs_cputime=%e\n', ...
              ks_cputime,  ks_cputime2,  eigs_cputime);  
     fprintf('ks_cpu/ks_cpu2=%e,    arpack_cpu/ks_cpu2=%e\n\n', speedgain4, speedgain3);
    
   end  
  
end

end
