%
% compare krylov_spectral.m with irbleigs.m and lobpcg.m on some relatively 
% realistic model problems.
%
% both irbleigs.m and lobpcg.m can be download online (e.g. via google search) 
% some of the sample mtx matrix should be download online (e.g. matrix market)
%

global A

for id = 1 : 5

    switch id
      case 1,
        A = model(42, 80); 
        
      case 2,
        A = model(41, 2000);

      case 3,
        A = model(43, 50, 50, 60 ); 

      case 4,
        A =mmread('s3dkt3m2.mtx');
        fprintf(' model: A = s3dkt3m2 \n')

      case 5,
        A=mmread('SiH4_sym.mtx');
        fprintf(' model: A = SiH4 \n')

      case 6,
        A=mmread('Si10H16_sym.mtx');
        fprintf(' model: A = Si10H16 \n')
    end
    
    ndim = size(A,1);

    %blkcontrol=-1;
    orthF = 0;
    max_restart = 2500;
    %tol = eps;
    tol = 1e-10;
    itt = 0;

    for nneed =  15  :  25  :  150
    %for nneed = 150 : 150:  600
        itt = itt+1;
        
        blkcontrol=0;  blksize=3;
        k = nneed + blksize;
        m = max(2*k + blksize, 50);

        F0 = rand(ndim, blksize);  
        F = F0;

        if (1 == 1)
            fprintf('\n=== begin krylov-spectral\n');
            V=[]; 
            opts = struct('tol', tol, 'blksize', blksize, 'maxvsize', m, 'nkeep', k);
            t1 = cputime;
            [eigval, V, nconv, T, info] = ks(nneed, F0, opts);
            ks_cputime(itt) = cputime - t1;

            fprintf('krylov-spectral:  ndim = %i,  blksize = %i,  nneed=%i, nconv=%i\n',...
                    ndim, blksize,  nneed, nconv);
            
            %% verify the actual residual norms
            F=A*V(:,1:nconv) - V(:,1:nconv)*diag(eigval(1:nconv));
            for i = 1:nconv
                errnrm(i) = norm(F(:,i))/abs(eigval(i));
            end
            fprintf('krylov-spectral:  actual residual norms  ||A*v-lambda*v||/|lambda|\n');
            fprintf('%11.5e   %11.5e   %11.5e   %11.5e   %11.5e\n', errnrm(1:nconv))
            [ernrm_max_ks, idx] = max(abs(errnrm(1:nconv)));
            fprintf('krylov-spectral max error norm:  errnrm( %i )=%e\n',idx,ernrm_max_ks);
        end
        fprintf('ks_cputime=%e, \n\n',  ks_cputime(itt))
        
        if (nneed < 350), 
            fprintf('\n=== begin irbleigs, may take a while if nneed > 50 ...\n');
            OPTS = struct('BLSZ', blksize, 'K', nneed, 'SIGMA', 'SE', 'V0', F0, ...
                          'MAXIT', max_restart, 'TOL', tol);
            V=[]; D=[];
            tBL = cputime;
            [V,D] = irbleigs(A, OPTS);
            irbl_cputime(itt) =cputime - tBL;
            
            nd = size(D,1);
            fprintf('\nirbleigs:  ndim = %i,   nneed=%i,   nconv=%i\n', ndim, nneed, nd);
            
            if (nd > 0),
                %% verify the actual residual norms
                F=A*V(:,1:nd) - V(:,1:nd)*D;
                for i = 1:nd
                    errnrm2(i) = norm(F(:,i))/abs(D(i,i));
                end
                fprintf('irbleigs:  actual residual norms  ||A*v-lambda*v||/lambda\n');
                fprintf('%11.5e   %11.5e   %11.5e   %11.5e   %11.5e\n', errnrm2(1:nd))
                [ernrm_max_irbl, idx] = max(abs(errnrm2(1:nd)));
                fprintf('irbleigs max error norm:  errnrm( %i )=%e\n\n',idx,ernrm_max_irbl);
                speedgain(itt)=irbl_cputime(itt)/ks_cputime(itt);
                fprintf('ks_max_errnrm=%e, irbl_max_errnrm=%e\n',ernrm_max_ks,ernrm_max_irbl);    
                fprintf('ks_cputime=%e,    irbl_cputime=%e,   (irbl_cpu/ks_cpu)=%e\n', ...
                        ks_cputime(itt), irbl_cputime(itt), speedgain(itt));  
                fprintf('\n');
                fprintf('eig diff:  eigvals - eig_irbl\n');
                fprintf('%11.5e   %11.5e   %11.5e   %11.5e   %11.5e\n',diag(D)-eigval(1:nd))
            else
                fprintf('irbleigs does not converge in %i iterations\n', max_restart);
            end
        end

        
        if (1==1),  
            fprintf('\n=== begin lobpcg, it can also take very long ...\n');
            
            V=[]; D=[];
            %OPTS=struct('issym',1,'isreal', 1,'tol', tol,'v0', F0,'maxit',max_restart);
            %[V,D] = lobpcg(A, nneed, 'SA', OPTS);
            tlob = cputime;
            V(:,1:nneed)=[F0, rand(ndim, nneed-blksize)];
            %[V,D] = lobpcg(V(:,1:nneed), A, nneed, tol, max_restart);
            [V,D] = lobpcg(V(:,1:nneed), A, tol, max_restart);
            lobpcg_cputime(itt) =cputime - tlob;
            
            nd = size(D,1); 
            fprintf('\nlobpcg:  ndim = %i,   nneed=%i,   nconv=%i\n', ndim, nneed, nd);
            
            if (nd > 0),
                %% verify the actual residual norms
                F=A*V(:,1:nd) - V(:,1:nd)*diag(D(1:nd));
                for i = 1:nd
                    errnrm4(i) = norm(F(:,i))/abs(D(i));
                end
                fprintf('lobpcg:  actual residual norms  ||A*v-lambda*v||/lambda\n');
                fprintf('%11.5e   %11.5e   %11.5e   %11.5e   %11.5e\n', errnrm4(1:nd))
                [ernrm_max_lobpcg, idx] = max(abs(errnrm4(1:nd)));
                fprintf('lobpcg max error norm:  errnrm( %i ) = %e\n\n', idx, ernrm_max_lobpcg);  
                speedgain1(itt)=lobpcg_cputime(itt)/ks_cputime(itt);
                fprintf('ks_max_errnrm=%e, lobpcg_max_errnrm=%e\n', ernrm_max_ks, ernrm_max_lobpcg);
                fprintf('ks_cputime=%e,    lobpcg_cputime=%e,   (lobpcg_cpu/ks_cpu)=%e\n', ...
                        ks_cputime(itt), lobpcg_cputime(itt), speedgain1(itt));  
                fprintf('\n');
                fprintf('eig diff:  eigvals - eig_lobpcg\n');
                fprintf('%11.5e   %11.5e   %11.5e   %11.5e   %11.5e\n', D-eigval(1:nd))
            else
                fprintf('lobpcg does not converge in %i iterations\n', max_restart);
            end
        end
        
        
        if (1==1),
            OPTS = struct('issym', 1, 'isreal', 1, 'tol', tol, 'v0', F0(:,1), ...
                          'maxit', max_restart, 'disp', 0);
            V=[]; D=[];
            teigs = cputime;
            [V,D] = eigs(A, nneed, 'SA', OPTS);
            eigs_cputime(itt) =cputime - teigs;
            
            nd = size(D,1);
            fprintf('\narpack eigs:  ndim = %i,   nneed=%i,   nconv=%i\n', ndim, nneed, nd);
            
            if (nd > 0),
                %
                % verify the actual residual norms
                %
                F=A*V(:,1:nd) - V(:,1:nd)*D;
                for i = 1:nd
                    errnrm3(i) = norm(F(:,i))/abs(D(i,i));
                end
                fprintf('arpack eigs:  actual residual norms  ||A*v-lambda*v||/|lambda|\n');
                fprintf('%11.5e   %11.5e   %11.5e   %11.5e   %11.5e\n', errnrm3(1:nd))
                [ernrm_max_arpack, idx] = max(abs(errnrm3(1:nconv)));
                fprintf('arpack max error norm:  errnrm( %i ) = %e\n\n', idx, ernrm_max_arpack);    
            else
                fprintf('arpack_eigs does not converge in %i iterations\n', max_restart);
            end

            speedgain2(itt)=eigs_cputime(itt)/ks_cputime(itt);
            fprintf('ks_max_errnrm=%e,     arpack_max_errnrm=%e\n', ...
                    ernrm_max_ks,  ernrm_max_arpack);  
            fprintf('ks_cputime=%e,     eigs_cputime=%e,   arpack_cpu/ks_cpu=%e\n\n', ...
                    ks_cputime(itt), eigs_cputime(itt), speedgain2(itt));  
            

            blksize = nneed;
            F = V(:,1:nneed)+ 5e-10*rand(ndim, nneed);
            %F = V(:,1:nneed);
            k = nneed;
            m = 2*k;  %min(2*k, floor(ndim/4));
            blkcontrol=-1;
            
            fprintf('\n ==>  begin krylov-spectral with good initial guesses\n');
            opts = struct('tol', tol, 'blksize', blksize, 'maxvsize', m, 'nkeep', k);
            t1 = cputime;
            [eigval, V, nconv, T, info] = ks(nneed, F, opts); 
            ks_cputime2(itt) = cputime - t1;

            fprintf('\nkrylov-spectral:  ndim = %i,   nneed=%i,   nconv=%i\n',...
                    ndim, nneed, nconv);
            
            %% verify the actual residual norms
            F=A*V(:,1:nconv) - V(:,1:nconv)*diag(eigval(1:nconv));
            for i = 1:nconv
                errnrm(i) = norm(F(:,i))/abs(eigval(i));
            end
            fprintf('krylov-spectral: actual residual norms ||A*v-lambda*v||/|lambda|\n');
            fprintf('%11.5e   %11.5e   %11.5e   %11.5e   %11.5e\n', errnrm(1:nconv))
            [ernrm_max_ks, idx] = max(abs(errnrm(1:nconv)));
            fprintf('krylov-spectral max error norm:  errnrm( %i ) = %e\n\n', ...
                    idx, ernrm_max_ks);
            
            speedgain3(itt) =  eigs_cputime(itt)/ ks_cputime2(itt);
            speedgain4(itt) =  ks_cputime(itt) / ks_cputime2(itt);
            fprintf('ks_cputime2=%e,   eigs_cputime=%e\n', ...
                    ks_cputime2(itt),  eigs_cputime(itt));  
            fprintf('(irbl_cpu/ks_cpu)=%e,  (lobpcg_cpu/ks_cpu)=%e \n',...
                    speedgain(itt), speedgain1(itt));   
            fprintf('(arpack_cpu/ks_cpu)=%e,  (arpack_cpu/ks_cpu2)=%e\n\n',...
                    speedgain2(itt), speedgain3(itt));
        end  
        
    end

end
